fn main() {
    println!("Hello, world!");
}

mod rover {
    #[derive(Debug, PartialEq, Copy, Clone)]
    pub enum Facing { N, E, S, W }

    type Rover = (i32, i32, Facing);

    fn velocity(direction : &Facing) -> (i32, i32) {
	match direction {
	    Facing::N  => ( 0, -1),
	    Facing::E  => ( 1,  0),
	    Facing::S  => ( 0,  1),
	    Facing::W  => (-1,  0)
	}
    }

    pub fn forward((lon, lat, direction) : &Rover) -> Rover {
	let (xoff, yoff) = velocity(&direction);
	(lon + xoff as i32, lat + yoff as i32, *direction)
    }

    pub fn backward((lon, lat, direction) : &Rover) -> Rover {
	let (xoff, yoff) = velocity(&direction);
	(lon - xoff as i32, lat - yoff as i32, *direction)
    }
}


#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn move_north() {
	let r = (1 as i32, 1 as i32, rover::Facing::N);
	assert_eq!(rover::forward(&r),
		   (1 as i32, 0 as i32, rover::Facing::N));

	let r = (1 as i32, 3 as i32, rover::Facing::N);
	assert_eq!(rover::forward(&r),
		   (1 as i32, 2 as i32, rover::Facing::N));
    }

    #[test]
    fn move_south() {
	let r = (1 as i32, 1 as i32, rover::Facing::S);
	assert_eq!(rover::forward(&r),
		   (1 as i32, 2 as i32, rover::Facing::S));
    }

    #[test]
    fn move_west() {
	let r = (1 as i32, 1 as i32, rover::Facing::W);
	assert_eq!(rover::forward(&r),
		   (0 as i32, 1 as i32, rover::Facing::W));
    }

    #[test]
    fn move_east() {
	let r = (1 as i32, 1 as i32, rover::Facing::E);
	assert_eq!(rover::forward(&r),
		   (2 as i32, 1 as i32, rover::Facing::E));
    }

    #[test]
    fn move_backward() {
	let r = (1 as i32, 1 as i32, rover::Facing::E);
	let r2 = rover::forward(&r);
	assert_eq!(rover::backward(&r2), r);
    }
}
